MQSeries Wrappers

MQSeries specialist (UK)	c Xephon



 IDENTIFICATION DIVISION.
 PROGRAM-ID. WRAPPER.
 ENVIRONMENT DIVISION.
 DATA DIVISION.
 WORKING-STORAGE SECTION.
     EXEC SQL DECLARE PMO_TAB TABLE   
     ( UNIQUEKEY                         CHAR(30),     
       STRUCID                           CHAR(4),      
       VERSION                           INTEGER,      
       OPTIONS                           INTEGER,      
       TIMEOUT                           INTEGER,       
       CONTEXT                           INTEGER,      
       KNOWNDESTCOUNT                    INTEGER,      
       UNKNOWNDESTCOUNT                  INTEGER,      
       INVALIDDESTCOUNT                  INTEGER,      
       RESOLVEDQNAME                     CHAR(48),     
       RESOLVEDQMGRNAME                  CHAR(48)      
     ) END-EXEC.                                    
* Example MQSeries Structure (MQPMO)   *
 01  MQPMO-ROW.
     05  MQPMO-UNIQUE-KEY                PIC X(48).
     05  PUT-MESSAGE-OPTIONS.
     EXEC SQL
         INCLUDE CMQPMOV
     END-EXEC.
 01  INITIALIZED-MQOD.
     COPY CMQODV. 
 01  INITIALIZED-MQMD.
     COPY CMQMDV. 
 01  MQSERIES-CONSTANTS.
     COPY CMQV.
 LINKAGE SECTION. 
 01  WRAPPER-CONTROL-DATA.
     05  WRAPPER-FUNCTION                PIC X(30).
     05  WRAPPER-QUEUE-MANAGER           PIC X(48).
     05  WRAPPER-MESSAGE-LENGTH          PIC S9(9) BINARY.
     05  WRAPPER-BUFFER-LENGTH           PIC S9(9) BINARY.
     05  WRAPPER-CONNECTION-HANDLE       PIC S9(9) BINARY.
     05  WRAPPER-OBJECT-HANDLE-OUT       PIC S9(9) BINARY.
     05  WRAPPER-OBJECT-HANDLE-IN        PIC S9(9) BINARY.
     05  WRAPPER-RETURN-CODE             PIC S9(9) BINARY.
     05  WRAPPER-MQSERIES-ERROR-DETAILS.
         10  WRAPPER-MQ-OPERATION        PIC X(12).
         10  WRAPPER-MQ-OBJECTNAME       PIC X(48).
         10  WRAPPER-MQ-COMPLETION-CODE  PIC S9(9) BINARY.
         10  WRAPPER-MQ-REASON-CODE      PIC S9(9) BINARY.
 01  WRAPPER-MQSERIES-STRUCTURES.
     05  NUMBER-OF-STRUCTURES  VALUE 6   PIC 9(4) BINARY.
     05  MQOD-NAME                       PIC X(48).
     05  MQOD-SQL-CODE                   PIC S9(9) BINARY.
     05  MQOO-NAME                       PIC X(48).
     05  MQOO-SQL-CODE                   PIC S9(9) BINARY.
     05  MQMD-NAME                       PIC X(48).
     05  MQMD-SQL-CODE                   PIC S9(9) BINARY.
     05  MQPMO-NAME                      PIC X(48).
     05  MQPMO-SQL-CODE                  PIC S9(9) BINARY.
     05  MQGMO-NAME                      PIC X(48).
     05  MQGMO-SQL-CODE                  PIC S9(9) BINARY.
     05  MQCO-NAME                       PIC X(48).
     05  MQCO-SQL-CODE                   PIC S9(9) BINARY.
 01  WRAPPER-MESSAGE-BUFFER              PIC X(1048576).
 PROCEDURE DIVISION USING WRAPPER-CONTROL-DATA
                          WRAPPER-MQSERIES-STRUCTURES
                          WRAPPER-MESSAGE-BUFFER.
 MAIN SECTION.
     PERFORM INITIALIZATION.
     PERFORM GET-MQSERIES-STRUCTURES.
     PERFORM PROCESS-REQUEST.
     GOBACK.
 INITIALIZATION SECTION.
* Set default values in MQSeries Structures. *
     INITIALIZE WRAPPER-MQSERIES-ERROR-DETAILS.
     MOVE INITIALIZED-MQOD TO MQOD-OUT
                              MQOD-IN.
     MOVE INITIALIZED-MQMD TO MQMD-OUT
                              MQMD-IN.
     MOVE MQOO-OUTPUT TO         OPEN-OPTIONS-OUT.
     MOVE MQOO-INPUT-AS-Q-DEF TO OPEN-OPTIONS-IN.
     MOVE MQCO-NONE TO CLOSE-OPTIONS-OUT
                       CLOSE-OPTIONS-IN.
 GET-MQSERIES-STRUCTURES SECTION.
* Retrieve named MQSeries Structures   *
     IF MQOD-NAME NOT = SPACES
         PERFORM RETRIEVE-MQOD
     END-IF.
     IF MQOO-NAME NOT = SPACES
         PERFORM RETRIEVE-MQOO
     END-IF.
     IF MQMD-NAME NOT = SPACES
         PERFORM RETRIEVE-MQMD
     END-IF.
     IF MQPMO-NAME NOT = SPACES
         PERFORM RETRIEVE-MQPMO
     END-IF.
     IF MQGMO-NAME NOT = SPACES
         PERFORM RETRIEVE-MQGMO
     END-IF.
     IF MQCO-NAME NOT = SPACES
         PERFORM RETRIEVE-MQCO
     END-IF.
 RETRIEVE-MQPMO SECTION.
* Retrieve the named Put Message Options. *
     EXEC SQL
         SELECT * INTO :MQPMO-UNIQUE-KEY,
                       :MQPMO-STRUCID,
                       :MQPMO-VERSION,
                       :MQPMO-OPTIONS,
                       :MQPMO-TIMEOUT,
                       :MQPMO-CONTEXT,
                       :MQPMO-KNOWNDESTCOUNT,
                       :MQPMO-UNKNOWNDESTCOUNT,
                       :MQPMO-INVALIDDESTCOUNT,
                       :MQPMO-RESOLVEDQNAME,
                       :MQPMO-RESOLVEDQMGRNAME
             FROM MQPMO_TAB D
                 WHERE D.UNIQUE-KEY = :MQPMO-NAME
     END-EXEC.
     MOVE SQLCODE TO MQPMO-SQL-CODE.
 PROCESS-REQUEST SECTION.
* Main logic                           *
     IF WRAPPER-FUNCTION = 'SEND'
     OR WRAPPER-FUNCTION = 'SEND FIRST'
     OR WRAPPER-FUNCTION = 'SEND RECEIVE'
         PERFORM CONNECT-QMGR
         PERFORM OPEN-QUEUE-OUT
         PERFORM PUT-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'SEND AGAIN'
     OR WRAPPER-FUNCTION = 'SEND LAST'
         PERFORM PUT-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'RECEIVE'
     OR WRAPPER-FUNCTION = 'RECEIVE FIRST'
         PERFORM CONNECT-QMGR
         PERFORM OPEN-QUEUE-IN
         PERFORM GET-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'RECEIVE AGAIN'
     OR WRAPPER-FUNCTION = 'RECEIVE LAST'
         PERFORM GET-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'SEND RECEIVE'
         PERFORM OPEN-QUEUE-IN
         PERFORM GET-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'COMMIT'
         PERFORM COMMIT-MESSAGE
     END-IF.
     IF WRAPPER-FUNCTION = 'SEND'
     OR WRAPPER-FUNCTION = 'SEND LAST'
     OR WRAPPER-FUNCTION = 'SEND RECEIVE'
         PERFORM CLOSE-QUEUE-OUT
     END-IF.
     IF WRAPPER-FUNCTION = 'RECEIVE' 
     OR WRAPPER-FUNCTION = 'RECEIVE LAST'
     OR WRAPPER-FUNCTION = 'SEND RECEIVE'
         PERFORM CLOSE-QUEUE-IN
     END-IF.
     IF WRAPPER-FUNCTION = 'SEND'
     OR WRAPPER-FUNCTION = 'SEND LAST'
     OR WRAPPER-FUNCTION = 'SEND RECEIVE'
     OR WRAPPER-FUNCTION = 'RECEIVE' 
     OR WRAPPER-FUNCTION = 'RECEIVE LAST'
         PERFORM DISCONNECT-QMGR
     END-IF.
 CONNECT-QMGR SECTION.
* Connect to Queue Manager             *
     CALL 'MQCONN' USING WRAPPER-QUEUE-MANAGER
                         WRAPPER-CONNECTION-HANDLE
                         WRAPPER-MQ-COMPLETION-CODE
                         WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO                    WRAPPER-RETURN-CODE
         MOVE 'CONNECT-QMGR' TO        WRAPPER-MQ-OPERATION
         MOVE WRAPPER-QUEUE-MANAGER TO WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 OPEN-QUEUE-OUT SECTION.
* Open output queue                   * 
     CALL 'MQOPEN' USING WRAPPER-CONNECTION-HANDLE
                         MQOD-OUT
                         OPEN-OPTIONS-OUT
                         WRAPPER-OBJECT-HANDLE-OUT
                         WRAPPER-MQ-COMPLETION-CODE
                         WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO              WRAPPER-RETURN-CODE
         MOVE 'OPEN-Q-OUT' TO    WRAPPER-MQ-OPERATION
         MOVE MQOD-OUT.MQOD-OBJECTNAME TO
                                 WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 PUT-MESSAGE SECTION.
* Put message onto queue               *
     CALL 'MQPUT' USING WRAPPER-CONNECTION-HANDLE
                        WRAPPER-OBJECT-HANDLE-OUT
                        MQMD-OUT
                        PUT-MESSAGE-OPTIONS
                        WRAPPER-MESSAGE-LENGTH
                        WRAPPER-MESSAGE-BUFFER
                        WRAPPER-MQ-COMPLETION-CODE
                        WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO              WRAPPER-RETURN-CODE
         MOVE 'PUT-MESSAGE' TO   WRAPPER-MQ-OPERATION
         MOVE MQOD-OUT.MQOD-OBJECTNAME TO
                                 WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 OPEN-QUEUE-IN SECTION.
* Open input queue           *
     CALL 'MQOPEN' USING WRAPPER-CONNECTION-HANDLE
                         MQOD-IN
                         OPEN-OPTIONS-IN
                         WRAPPER-OBJECT-HANDLE-IN
                         WRAPPER-MQ-COMPLETION-CODE
                         WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO              WRAPPER-RETURN-CODE
         MOVE 'OPEN-Q-IN' TO     WRAPPER-MQ-OPERATION
         MOVE MQOD-IN.MQOD-OBJECTNAME TO
                                 WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 GET-MESSAGE SECTION.
* Get message                  *
     CALL 'MQGET' USING WRAPPER-CONNECTION-HANDLE
                        WRAPPER-OBJECT-HANDLE-IN
                        MQMD-IN
                        GET-MESSAGE-OPTIONS
                        WRAPPER-BUFFER-LENGTH
                        WRAPPER-MESSAGE-BUFFER
                        WRAPPER-MESSAGE-LENGTH
                        WRAPPER-MQ-COMPLETION-CODE
                        WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO              WRAPPER-RETURN-CODE
         MOVE 'GET-MESSAGE' TO   WRAPPER-MQ-OPERATION
         MOVE MQOD-IN.MQOD-OBJECTNAME TO
                                 WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 COMMIT-MESSAGE SECTION.
* Commit the put / got message           *
     CALL 'MQCMIT' USING WRAPPER-CONNECTION-HANDLE
                         WRAPPER-MQ-COMPLETION-CODE
                         WRAPPER-MQ-REASON-CODE
     END-CALL.
     IF WRAPPER-MQ-COMPLETION-CODE NOT = MQCC-OK
         MOVE +8 TO                    WRAPPER-RETURN-CODE
         MOVE 'CMIT-MESSAGE' TO        WRAPPER-MQ-OPERATION
         MOVE WRAPPER-QUEUE-MANAGER TO WRAPPER-MQ-OBJECTNAME
         GOBACK
     END-IF.
 CLOSE-QUEUE-OUT SECTION.

